#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import math
import time
import numpy as np
import matplotlib.pyplot as plt

sys.path.append('gridmap')
sys.path.append('gridplanner')
sys.path.append('game')
sys.path.append('player')

import GridMap as gmap
import Game as gm
import Player as pl
import pickle

TIMEOUT = 1.0

if __name__ == "__main__":

    # define games:
    #  problem/map name 
    #  game definition file
    games = [        
         ("grid", "games/grid_mc_evader_1.game"),
         ("grid", "games/grid_mc_pursuer_1.game"),
         ("grid", "games/grid_mc_pursuer_2.game"),
         ("pacman_small", "games/pacman_small_mc_evader_1.game"),
         ("pacman_small", "games/pacman_small_mc_pursuer_1.game"),
         ("pacman_small", "games/pacman_small_mc_pursuer_2.game"),
         ("pacman_small", "games/for_students_evader.game"),
         ("pacman_small", "games/for_students_pursuers.game"),
         ("pacman", "games/pacman_mc_evader_1.game"),
         ("pacman", "games/pacman_mc_pursuer_1.game"),
         ("pacman", "games/pacman_mc_pursuer_2.game"),         
    ]

    # fetch individual scenarios
    for game in games:

        # instantiate the map
        gridmap = gmap.GridMap()

        # load map from file
        gridmap.load_map("maps/" + game[0] + ".csv")

        # load the game configuration
        players = []

        with open(game[0] + ".dist", "rb") as distance_matrix_file:
            distance_matrix = pickle.load(distance_matrix_file)

        with open(game[1]) as fp:
            for line in fp:
                if line[0] == "#":  # skip comments
                    continue
                q = line.split()

                role = q[0]  # player role
                policy = q[1]  # player policy
                color = q[2]  # player color
                robots = []  # list of player's robots
                for x in range(3, len(q), 2):
                    robots.append((int(q[x]), int(q[x + 1])))
                
                players.append(pl.Player(robots, role, policy=policy, color=color, timeout=TIMEOUT, game=game[0],
                                             distance_matrix=distance_matrix))
                                             
                print(role, "is", policy)
        # instantiate the game
        pursuit_game = gm.Game(gridmap, players)
        pursuit_game.plot_game()
        pursuit_game.plot_pause(0.1)

        ######################################
        # Simulate the game
        ######################################

        # simulate game of n_steps
        n_steps = 15

        # comment this line to turn off saving images
        pursuit_game.save_plot("images/" + game[0] + "0.png")

        for i in range(0, n_steps):
            # if not all the evaders have been captured
            if not pursuit_game.is_end():
                # make a turn                
                pursuit_game.step()

                # plot the game result
                pursuit_game.plot_game()
                pursuit_game.plot_pause(0.01)
                # comment this line to turn off saving images
                pursuit_game.save_plot("images/" + game[0] + str(i + 1) + ".png")

        if not pursuit_game.is_end():
            print("Evader escaped")
        else:
            print("GAME OVER")
        print()
        # waiting for end
        pursuit_game.plot_pause(2)
